﻿/*
  Antenna House PDF Tool API V7.0
  .Net Interface sample program

  概要：公開鍵を用いてPDFを暗号化する際に全ての権限で許可を出す
  (X.509ファイルで暗号化)

  Copyright (C) 2023- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

using PdfTkNet;
using System;
using System.IO;

namespace EncryptSetPubKeySetFullPermission
{
    internal class EncryptSetPubKeySetFullPermission
    {
        // そのクラスのusageを表示する関数
        private static void printUsage()
        {
            Console.Write("usage: EncryptSetPubKeyWithX509 in-pdf-file out-pdf-file ");
            Console.WriteLine("暗号化種類 X509Pubkey-path set-full-permission");
            Console.WriteLine("暗号化種類");
            Console.WriteLine("0 : 128 bit RC4  1 : 128 bit AES  2 : 256 bit AES");
            Console.WriteLine("set-full-permission");
            Console.WriteLine("0 : デフォルトのアクセス権限を設定する。  1 : 全ての権限を許可する");
        }

        static void Main(string[] args)
        {
            if (args.Length < 4)
            {
                printUsage();
                return;
            }

            // コマンドライン引数の取得
            int kind;
            bool setFullPerm;
            try
            {
                kind = int.Parse(args[2]);
                switch (kind)
                {
                    case 0:
                    case 1:
                    case 2:
                        break;
                    default:
                        Console.WriteLine("暗号化種類には、0から2の数値を指定してください。");
                        printUsage(); // usageメッセージの表示
                        return;
                }
            }
            catch (FormatException numfe)
            {
                Console.WriteLine("ERROR!!暗号化種類の文字形式が正しくありません。");
                Console.WriteLine("暗号化種類には、0から2の数値を指定してください。");
                Console.WriteLine(numfe.StackTrace);
                printUsage(); // usageメッセージの表示
                return;
            }
            try
            {
                setFullPerm = readBoolArgs(args[4], "set-full-permissionは" +
                                               "0か1で指定してください。");
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine(ex.Message);
                printUsage(); // usageメッセージの表示
                return;
            }
            try
            {
                using (PtlParamInput inputFile = new PtlParamInput(args[0]))
                using (PtlParamOutput outputFile = new PtlParamOutput(args[1]))
                using (PtlParamInput x509Pubkey = new PtlParamInput(args[3]))
                using (PtlPDFDocument doc = new PtlPDFDocument())
                {
                    // PDFファイルをロードします
                    doc.load(inputFile);

                    // PDFにX.509形式の証明書を設定します
                    switch (kind)
                    {
                        case 0:
                            // 128 bit RC4
                            Console.WriteLine("128bit RC4で暗号化します。");
                            encryptX509PubkeySetFullPerm128RC4(doc, x509Pubkey, setFullPerm);
                            break;
                        case 1:
                            // 128 bit AES
                            Console.WriteLine("128bit AESで暗号化します。");
                            encryptX509PubkeySetFullPerm128AES(doc, x509Pubkey, setFullPerm);
                            break;
                        case 2:
                            // 256 bit AES
                            Console.WriteLine("256bit AESで暗号化します。");
                            encryptX509PubkeySetFullPerm256AES(doc, x509Pubkey, setFullPerm);
                            break;
                    }
                    // ファイルに保存します
                    doc.save(outputFile);
                }
            }
            catch (PtlException pex)
            {
                Console.WriteLine("PtlException : ErrorCode = " + pex.getErrorCode() +
                                   "\n  " + pex.getErrorMessage());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
            finally
            {
                Console.WriteLine("-- 完了 --");
            }
        }

        private static void encryptX509PubkeySetFullPerm128RC4(PtlPDFDocument doc, PtlParamInput x509Pubkey, bool setFullPerm)
        {
            try
            {
                using (PtlEncryptPubKey128RC4 encPubkey128RC4 = new PtlEncryptPubKey128RC4())
                {//256ビットAES暗号化された公開キーセキュリティハンドラを表すクラス
                    encryptX509PubkeySetFullPerm(doc, x509Pubkey, encPubkey128RC4, setFullPerm);
                }
            }
            catch (IOException ex)
            {
                throw ex;
            }
            catch (PtlException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private static void encryptX509PubkeySetFullPerm128AES(PtlPDFDocument doc, PtlParamInput x509Pubkey, bool setFullPerm)
        {
            try
            {
                using (PtlEncryptPubKey128AES encPubkey128AES = new PtlEncryptPubKey128AES())
                {//256ビットAES暗号化された公開キーセキュリティハンドラを表すクラス
                    encryptX509PubkeySetFullPerm(doc, x509Pubkey, encPubkey128AES, setFullPerm);
                }
            }
            catch (IOException ex)
            {
                throw ex;
            }
            catch (PtlException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private static void encryptX509PubkeySetFullPerm256AES(PtlPDFDocument doc, PtlParamInput x509Pubkey, bool setFullPerm)
        {
            try
            {
                using (PtlEncryptPubKey256AES encPubkey256AES = new PtlEncryptPubKey256AES())
                {//256ビットAES暗号化された公開キーセキュリティハンドラを表すクラス
                    encryptX509PubkeySetFullPerm(doc, x509Pubkey, encPubkey256AES, setFullPerm);
                }
            }
            catch (IOException ex)
            {
                throw ex;
            }
            catch (PtlException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private static void encryptX509PubkeySetFullPerm(PtlPDFDocument doc, PtlParamInput x509Pubkey,
                                              PtlEncryptPubKey encPubkey, bool setFullPerm)
        {
            try
            {
                using (PtlRecipients recipients = encPubkey.getRecipients())   //証明書受信者のコンテナを表したクラス
                using (PtlRecipient recipient = new PtlRecipient())            //証明書の受信者を表したクラス
                using (PtlEncryptPermissionPubKey permission = new PtlEncryptPermissionPubKey())
                {
                    //すべての編集権限を与えるかどうかを設定
                    permission.setFullPermission(setFullPerm);

                    //受信者クラスにX.509形式の証明書を設定
                    recipient.setX509(x509Pubkey);
                    //受信者クラスにアクセス許可フラグを設定
                    recipient.setPermission(permission);
                    //受信者コンテナのクラスに受信者クラスを追加
                    recipients.append(recipient);

                    //以下は標準セキュリティハンドラと共通の設定
                    //すべてのコンテンツを暗号化する
                    encPubkey.setEncryptComponent(PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_ALL);

                    // 設定したセキュリティハンドラを用いてPDFを暗号化
                    doc.setEncrypt(encPubkey);
                }
            }
            catch (IOException ex)
            {
                throw ex;
            }
            catch (PtlException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /**
         * 0または1を入力されたargsにより、trueまたはfalseを返すメソッド。
         * 
         * @param args 与えられるコマンドライン引数。0または1でtrueまたはfalseを指定する。
         * @param errorMessage argsが0か1でなかった場合に出力されるエラーメッセージを指定する。
         * @return argsの数値を読み取った結果を戻す
         * @throws ArgumentException argsが0か1でなかった場合に発生。
         */
        public static bool readBoolArgs(String args, String errorMessage)
        {
            bool trueOrFalse = false;
            try
            {
                // argsの読み取り
                switch (args)
                {
                    case "0":
                        trueOrFalse = false;
                        break;
                    case "1":
                        trueOrFalse = true;
                        break;
                    default:
                        throw new ArgumentException(errorMessage);
                }
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }

            return trueOrFalse;
        }
    }
}
