﻿/*
  Antenna House PDF Tool API V7.0
  .Net Interface sample program

  概要：座標ベースでのテキスト抽出の際に異なるテキストが
        どの程度重なり合っていたら除外するかを指定します。

  Copyright (C) 2023- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

using PdfTkNet;
using System;
using System.IO;

namespace ExtTextSetDiffTextOmitRatio
{
    internal class ExtTextSetDiffTextOmitRatio
    {
        // そのクラスのusageを表示する関数
        private static void printUsage()
        {
            Console.WriteLine("usage: ExtTextSetDiffTextOmitRatio"
                             + " in-pdf-file out-text-file"
                             + " page-to-extract diffText-omit-ratio"
                             + " omit-diffText-in-diffColor");
            Console.WriteLine("diffText-omit-ratio:");
            Console.WriteLine("異なる文字が重なり合っていた場合にどの程度重なり合っていたら除外するかを小数で指定");
            Console.WriteLine("omit-diffText-only-diffColor:");
            Console.WriteLine("0:重なった異なる文字の色が同じ場合も取り除く 1:重なった異なる文字の色が違う場合だけ取り除く");
        }

        static void Main(string[] args)
        {
            if (args.Length < 5)
            {
                printUsage(); // usageメッセージの表示
                return;
            }

            // コマンドライン引数の読み取り・判定
            // 出力PDFの名前はあとで渡すためにString型で保存する。
            String outputTextURI = args[1];
            int pageToExtract = int.Parse(args[2]);
            float diffTextOmitRatio = float.Parse(args[3]);
            bool omitDiffTextOnlyInDiffColor = true;
            try
            {
                omitDiffTextOnlyInDiffColor = readBoolArgs(args[4],
                                           "omit-diffText-only-diffColorは1か0で指定してください。");
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine(ex.Message);
                printUsage(); // usageメッセージの表示
                return;
            }

            try
            {
                using (PtlParamInput inputFile = new PtlParamInput(args[0]))
                using (PtlPDFDocument doc = new PtlPDFDocument())
                {
                    // PDFファイルをロード
                    doc.load(inputFile);
                    //ページ数を取得
                    int wholePageNum = doc.getPageCount();
                    //pageToExtractのエラー処理
                    if (wholePageNum < pageToExtract)
                    {
                        Console.WriteLine("ERROR: page-to-extractはPDFの総ページ数より小さい値を指定してください。");
                        Console.WriteLine("総ページ数：" + wholePageNum);
                        printUsage();
                        return;
                    }
                    //ページコンテナの取得
                    using (PtlPages pages = doc.getPages())
                    {
                        // ページコンテナが空かどうか
                        if (pages.isEmpty())
                        {
                            Console.WriteLine("ERROR : ページコンテナが空");
                            return;
                        }
                        // ページの取得(パラメータindexは0が先頭のため1を引く)
                        using (PtlPage page = pages.get(pageToExtract - 1))
                        // ページコンテントの取得
                        using (PtlContent content = page.getContent())
                        // 文字抽出のパラメータクラス。
                        using (PtlParamExtractText paramExtractText = new PtlParamExtractText())
                        {
                            // 異なるテキストがどのぐらい重なっていたら取り除くかを指定する。
                            paramExtractText.setDifferentTextOmitRatio(diffTextOmitRatio,
                                                                       omitDiffTextOnlyInDiffColor);

                            // 抽出するテキストを座標順に並べるよう指定
                            //（setSameTextOmitRatioのためには指定が必要）
                            paramExtractText.setTextType(PtlParamExtractText.TEXT_TYPE.TEXT_SORT);

                            // 文字列抽出
                            String TextFromPdf = content.extractText(paramExtractText);
                            // 文字列の出力
                            File.WriteAllText(outputTextURI, TextFromPdf);
                            //outputTextFile(outputTextURI, TextFromPdf);
                            Console.WriteLine(TextFromPdf);
                        }
                    }
                }
            }
            catch (PtlException pex)
            {
                Console.WriteLine("PtlException : ErrorCode = " + pex.getErrorCode() +
                                   "\n  " + pex.getErrorMessage());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
            finally
            {
                Console.WriteLine("-- 完了 --");
            }
        }

        /**
         * 0または1を入力されたargsにより、trueまたはfalseを返すメソッド。
         * 
         * @param args 与えられるコマンドライン引数。0または1でtrueまたはfalseを指定する。
         * @param errorMessage argsが0か1でなかった場合に出力されるエラーメッセージを指定する。
         * @return argsの数値を読み取った結果を戻す
         * @throws ArgumentException argsが0か1でなかった場合に発生。
         */
        public static bool readBoolArgs(String args, String errorMessage)
        {
            bool trueOrFalse = false;
            try
            {
                // argsの読み取り
                switch (args)
                {
                    case "0":
                        trueOrFalse = false;
                        break;
                    case "1":
                        trueOrFalse = true;
                        break;
                    default:
                        throw new ArgumentException(errorMessage);
                }
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }

            return trueOrFalse;
        }
    }
}
