﻿/*
  Antenna House PDF Tool API V7.0
  .Net Interface sample program

  概要：FDFに保存されたフォームフィールドデータを指定したPDFにインポートします。
        その後そのPDFを指定した名前で出力します。

  Copyright (C) 2023- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

using PdfTkNet;
using System;

namespace ImportFormFieldsFromFDF
{
    internal class ImportFormFieldsFromFDF
    {
        // そのクラスのusageを表示する関数
        private static void printUsage()
        {
            Console.Write("usage: ImportFormFieldsFromFDF in-pdf-file");
            Console.WriteLine(" out-pdf-file fdf-file-to-import");
        }

        static void Main(string[] args)
        {
            if (args.Length < 3)
            {
                printUsage();
                return;
            }

            // コマンドライン引数の取得
            try
            {
                using (PtlParamInput inputFile = new PtlParamInput(args[0]))
                using (PtlParamOutput outputFile = new PtlParamOutput(args[1]))
                using (PtlParamInput FdfToImport = new PtlParamInput(args[2]))
                using (PtlPDFDocument doc = new PtlPDFDocument())
                {
                    // PDFファイルをロードします。
                    doc.load(inputFile);
                    //ページコンテナの取得
                    using (PtlPages pages = doc.getPages())
                    {
                     // ページコンテナが空かどうか
                        if (pages.isEmpty())
                        {
                            Console.WriteLine("ページコンテナが空です。処理を終了します。");
                            return;
                        }
                    }
                    // FDFファイルからフォームフィールドデータをインポートする
                    //（設定できなかったフォームフィールドの値を取得する）
                    using (PtlFormFieldValues failedFormFieldValues = doc.importFormFieldsFromFDF(FdfToImport))
                    {
                        getFailedFormFieldInfo(failedFormFieldValues);
                        Console.WriteLine("フォームフィールドデータをインポートしました。");
                    }
                    // ファイルに保存します。
                    doc.save(outputFile);
                }
            }
            catch (PtlException pex)
            {
                Console.WriteLine("PtlException : ErrorCode = " + pex.getErrorCode() +
                                   "\n  " + pex.getErrorMessage());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
            finally
            {
                Console.WriteLine("-- 完了 --");
            }
        }

        private static void getFailedFormFieldInfo(PtlFormFieldValues formFieldValues)
        {
            try
            {
                int numOfFormFieldValues = formFieldValues.getCount();
                Console.WriteLine("設定できなかったフォームフィールドの数は " + numOfFormFieldValues + " 個です。");
                //0個だった場合は処理を抜ける。
                if (numOfFormFieldValues == 0)
                {
                    return;
                }

                Console.WriteLine("設定できなかったフォームフィールドの情報を表示します。");
                Console.WriteLine("---");

                // 0個でなかった場合は処理を続け、情報を得る
                for (int i = 0; i < numOfFormFieldValues; i++)
                {
                    Console.WriteLine("-");
                    Console.WriteLine("index No. " + i);
                    using (PtlFormFieldValue formFieldValue = formFieldValues.get(i))
                    {
                        Console.WriteLine("  フォームフィールド名 : " + formFieldValue.getFieldName());
                        Console.WriteLine("  フォームフィールド値 : " + formFieldValue.getValue());
                    }
                }
                Console.WriteLine("---");
            }
            catch (PtlException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}
